@@ -9,6 +9,8 @@
 #
 from typing import cast, Any, Callable, Iterator, List, Optional, Tuple, Union
 
+from elementpath.datatypes import AnyAtomicType
+
 from ..exceptions import XMLSchemaValueError
 from ..names import XSD_GROUP, XSD_ATTRIBUTE_GROUP, XSD_SEQUENCE, XSD_OVERRIDE, \
     XSD_ALL, XSD_CHOICE, XSD_ANY_ATTRIBUTE, XSD_ATTRIBUTE, XSD_COMPLEX_CONTENT, \
@@ -519,14 +521,6 @@ class XsdComplexType(XsdType, ValidationMixin[Union[ElementType, str, bytes], An
     def model_group(self) -> Optional[XsdGroup]:
         return self.content if isinstance(self.content, XsdGroup) else None
 
-    @property
-    def content_type(self) -> Union[XsdGroup, XsdSimpleType]:
-        """Property that returns the attribute *content*, for backward compatibility."""
-        import warnings
-        warnings.warn("'content_type' attribute has been replaced by 'content' "
-                      "and will be removed in version 2.0", DeprecationWarning, stacklevel=2)
-        return self.content
-
     @property
     def content_type_label(self) -> str:
         if self.is_empty():
@@ -765,8 +759,16 @@ class XsdComplexType(XsdType, ValidationMixin[Union[ElementType, str, bytes], An
             name = obj.name
             value = obj
 
-        xsd_element = self.schema.create_element(name, parent=self, form='unqualified')
-        xsd_element.type = self
+        xsd_type: BaseXsdType
+        if isinstance(value, list):
+            xsd_type = self.any_simple_type
+        elif isinstance(value, AnyAtomicType):
+            xsd_type = self.any_atomic_type
+        else:
+            xsd_type = self.any_type
+
+        xsd_element = self.schema.create_element(name, parent=xsd_type, form='unqualified')
+        xsd_element.type = xsd_type
         yield from xsd_element.iter_encode(value, validation, **kwargs)
 
 
