@@ -11,17 +11,20 @@
 This module contains classes for XML Schema elements, complex types and model groups.
 """
 import warnings
+from copy import copy as _copy
 from decimal import Decimal
 from types import GeneratorType
 from typing import TYPE_CHECKING, cast, Any, Dict, Iterator, List, Optional, Tuple, Type, Union
-from elementpath import XPath2Parser, ElementPathError, XPathContext, XPathToken
+from xml.etree.ElementTree import Element
+
+from elementpath import XPath2Parser, ElementPathError, XPathContext, XPathToken, \
+    ElementNode, LazyElementNode, SchemaElementNode, build_schema_node_tree
 from elementpath.datatypes import AbstractDateTime, Duration, AbstractBinary
 
 from ..exceptions import XMLSchemaTypeError, XMLSchemaValueError
 from ..names import XSD_COMPLEX_TYPE, XSD_SIMPLE_TYPE, XSD_ALTERNATIVE, \
     XSD_ELEMENT, XSD_ANY_TYPE, XSD_UNIQUE, XSD_KEY, XSD_KEYREF, XSI_NIL, \
     XSI_TYPE, XSD_ERROR, XSD_NOTATION_TYPE
-from ..etree import ElementData, etree_element
 from ..aliases import ElementType, SchemaType, BaseXsdType, SchemaElementType, \
     ModelParticleType, ComponentClassType, AtomicValueType, DecodeType, \
     IterDecodeType, IterEncodeType
@@ -29,16 +32,17 @@ from ..translation import gettext as _
 from ..helpers import get_qname, get_namespace, etree_iter_location_hints, \
     raw_xml_encode, strictly_equal
 from .. import dataobjects
-from ..converters import XMLSchemaConverter
-from ..xpath import XMLSchemaProtocol, ElementProtocol, XMLSchemaProxy, \
+from ..converters import ElementData, XMLSchemaConverter
+from ..xpath import XsdSchemaProtocol, XsdElementProtocol, XMLSchemaProxy, \
     ElementPathMixin, XPathElement
+from ..resources import XMLResource
 
 from .exceptions import XMLSchemaValidationError, XMLSchemaTypeTableWarning
 from .helpers import get_xsd_derivation_attribute
 from .xsdbase import XSD_TYPE_DERIVATIONS, XSD_ELEMENT_DERIVATIONS, \
     XsdComponent, ValidationMixin
 from .particles import ParticleMixin, OccursCalculator
-from .identities import IdentityXPathContext, XsdIdentity, XsdKey, XsdUnique, \
+from .identities import XsdIdentity, XsdKey, XsdUnique, \
     XsdKeyref, IdentityCounter, IdentityCounterType
 from .simple_types import XsdSimpleType
 from .attributes import XsdAttribute
@@ -109,7 +113,7 @@ class XsdElement(XsdComponent, ParticleMixin,
 
     binding: Optional[DataBindingType] = None
 
-    def __init__(self, elem: etree_element,
+    def __init__(self, elem: Element,
                  schema: SchemaType,
                  parent: Optional[XsdComponent] = None,
                  build: bool = True) -> None:
@@ -384,13 +388,26 @@ class XsdElement(XsdComponent, ParticleMixin,
     @property
     def xpath_proxy(self) -> XMLSchemaProxy:
         return XMLSchemaProxy(
-            schema=cast(XMLSchemaProtocol, self.schema),
-            base_element=cast(ElementProtocol, self)
+            schema=cast(XsdSchemaProtocol, self.schema),
+            base_element=cast(XsdElementProtocol, self)
+        )
+
+    @property
+    def xpath_node(self) -> SchemaElementNode:
+        schema_node = self.schema.xpath_node
+        node = schema_node.get_element_node(cast(XsdElementProtocol, self))
+        if isinstance(node, SchemaElementNode):
+            return node
+
+        return build_schema_node_tree(
+            root=cast(XsdElementProtocol, self),
+            elements=schema_node.elements,
+            global_elements=schema_node.children,
         )
 
     def build(self) -> None:
         if self._build:
-            return
+            return None
         self._build = True
         self._parse()
 
@@ -530,6 +547,8 @@ class XsdElement(XsdComponent, ParticleMixin,
         if text is None:
             text = self.fixed if self.fixed is not None else self.default
             if text is None:
+                if self.type.is_valid(''):
+                    self.type.text_decode('')
                 return None
         return self.type.text_decode(text)
 
@@ -659,9 +678,11 @@ class XsdElement(XsdComponent, ParticleMixin,
                         if isinstance(identity.elements, tuple):
                             continue  # Skip unbuilt identities
 
-                        context = IdentityXPathContext(
-                            self.schema, item=xpath_element  # type: ignore[arg-type]
+                        context = XPathContext(
+                            root=self.schema.xpath_node,
+                            item=cast(XsdElementProtocol, xpath_element)
                         )
+
                         for e in identity.selector.token.select_results(context):
                             if not isinstance(e, XsdElement):
                                 reason = _("selector xpath expression can only select elements")
@@ -704,7 +725,7 @@ class XsdElement(XsdComponent, ParticleMixin,
             if not self.nillable:
                 reason = _("element is not nillable")
                 yield self.validation_error(validation, reason, obj, **kwargs)
-            elif xsi_nil not in {'0', '1', 'false', 'true'}:
+            elif xsi_nil not in ('0', '1', 'false', 'true'):
                 reason = _("xsi:nil attribute must have a boolean value")
                 yield self.validation_error(validation, reason, obj, **kwargs)
             elif xsi_nil in ('0', 'false'):
@@ -751,7 +772,7 @@ class XsdElement(XsdComponent, ParticleMixin,
 
             text = obj.text
             if self.fixed is not None:
-                if text is None:
+                if not text:
                     text = self.fixed
                 elif text == self.fixed:
                     pass
@@ -782,20 +803,13 @@ class XsdElement(XsdComponent, ParticleMixin,
                     msg = _("missing enumeration facet in xs:NOTATION subtype")
                     yield self.validation_error(validation, msg, text, **kwargs)
 
-            if text is None:
-                for result in content_decoder.iter_decode('', validation, **kwargs):
-                    if isinstance(result, XMLSchemaValidationError):
-                        yield self.validation_error(validation, result, obj, **kwargs)
-                        if 'filler' in kwargs:
-                            value = kwargs['filler'](self)
-            else:
-                for result in content_decoder.iter_decode(text, validation, **kwargs):
-                    if isinstance(result, XMLSchemaValidationError):
-                        yield self.validation_error(validation, result, obj, **kwargs)
-                    elif result is None and 'filler' in kwargs:
-                        value = kwargs['filler'](self)
-                    else:
-                        value = result
+            for result in content_decoder.iter_decode(text or '', validation, **kwargs):
+                if isinstance(result, XMLSchemaValidationError):
+                    yield self.validation_error(validation, result, obj, **kwargs)
+                elif result is None and 'filler' in kwargs:
+                    value = kwargs['filler'](self)
+                elif text or kwargs.get('keep_empty'):
+                    value = result
 
             if 'value_hook' in kwargs:
                 value = kwargs['value_hook'](value, xsd_type)
@@ -818,6 +832,9 @@ class XsdElement(XsdComponent, ParticleMixin,
 
         if converter is not None:
             element_data = ElementData(obj.tag, value, content, attributes)
+            if 'element_hook' in kwargs:
+                element_data = kwargs['element_hook'](element_data, self, xsd_type)
+
             try:
                 yield converter.element_decode(element_data, self, xsd_type, level)
             except (ValueError, TypeError) as err:
@@ -828,7 +845,9 @@ class XsdElement(XsdComponent, ParticleMixin,
         if content is not None:
             del content
 
-        # Collects fields values for identities that refer to this element.
+        element_node: Union[None, ElementNode, LazyElementNode] = None
+
+        # Collect field values for identities that refer to this element.
         for identity, counter in identities.items():
             if not counter.enabled or not identity.elements:
                 continue
@@ -844,17 +863,26 @@ class XsdElement(XsdComponent, ParticleMixin,
                 if xsd_type is self.type:
                     xsd_fields = identity.elements[xsd_element]
                     if xsd_fields is None:
-                        xsd_fields = identity.get_fields(xsd_element)
+                        xsd_fields = identity.get_fields(xsd_element.xpath_node)
                         identity.elements[xsd_element] = xsd_fields
                 else:
-                    xsd_element = cast(XsdElement, self.copy())
+                    xsd_element = _copy(self)
                     xsd_element.type = xsd_type
-                    xsd_fields = identity.get_fields(xsd_element)
+                    xsd_fields = identity.get_fields(xsd_element.xpath_node)
 
                 if all(x is None for x in xsd_fields):
                     continue
                 decoders = cast(Tuple[XsdAttribute, ...], xsd_fields)
-                fields = identity.get_fields(obj, namespaces, decoders=decoders)
+
+                if element_node is None:
+                    try:
+                        resource = cast(XMLResource, kwargs['source'])
+                    except KeyError:
+                        element_node = LazyElementNode(obj, nsmap=_copy(namespaces))
+                    else:
+                        element_node = resource.get_xpath_node(obj)
+
+                fields = identity.get_fields(element_node, namespaces, decoders=decoders)
             except (XMLSchemaValueError, XMLSchemaTypeError) as err:
                 yield self.validation_error(validation, err, obj, **kwargs)
             else:
@@ -960,7 +988,7 @@ class XsdElement(XsdComponent, ParticleMixin,
                 if default_namespace and not isinstance(xsd_type, XsdSimpleType):
                     # Adjust attributes mapped into default namespace
 
-                    ns_part = '{%s}' % default_namespace
+                    ns_part = f'{{{default_namespace}}}'
                     for k in list(element_data.attributes):
                         if not k.startswith(ns_part):
                             continue
@@ -990,7 +1018,7 @@ class XsdElement(XsdComponent, ParticleMixin,
                 pass
             elif self.fixed is not None:
                 errors.append("xsi:nil='true' but the element has a fixed value.")
-            elif element_data.text is not None or element_data.content:
+            elif element_data.text not in (None, '') or element_data.content:
                 errors.append("xsi:nil='true' but the element is not empty.")
             else:
                 elem = converter.etree_element(element_data.tag, attrib=attributes, level=level)
@@ -1049,7 +1077,7 @@ class XsdElement(XsdComponent, ParticleMixin,
         if not name:
             return False
         elif default_namespace and name[0] != '{':
-            qname = '{%s}%s' % (default_namespace, name)
+            qname = f'{{{default_namespace}}}{name}'
             if name == self.name or qname == self.name:
                 return True
             return any(name == e.name or qname == e.name for e in self.iter_substitutes())
@@ -1063,7 +1091,7 @@ class XsdElement(XsdComponent, ParticleMixin,
         if not name:
             return None
         elif default_namespace and name[0] != '{':
-            qname = '{%s}%s' % (default_namespace, name)
+            qname = f'{{{default_namespace}}}{name}'
             if name == self.name or qname == self.name:
                 return self
 
@@ -1094,8 +1122,8 @@ class XsdElement(XsdComponent, ParticleMixin,
                         other.min_occurs != other.max_occurs and \
                         self.max_occurs != 0 and not other.abstract \
                         and self.xsd_version == '1.0':
-                    # An UPA violation case. Base is the head element, it's not
-                    # abstract and has non deterministic occurs: this is less
+                    # A UPA violation case. Base is the head element, it's not
+                    # abstract and has non-deterministic occurs: this is less
                     # restrictive than W3C test group (elemZ026), marked as
                     # invalid despite it's based on an abstract declaration.
                     # See also test case invalid_restrictions1.xsd.
@@ -1109,6 +1137,8 @@ class XsdElement(XsdComponent, ParticleMixin,
 
             if check_occurs and not self.has_occurs_restriction(other):
                 return False
+            elif self.max_occurs == 0 and check_occurs:
+                return True  # type is not effective if the element can't have occurrences
             elif not self.is_consistent(other) and self.type.elem is not other.type.elem and \
                     not self.type.is_derived(other.type, 'restriction') and not other.type.abstract:
                 return False
@@ -1306,12 +1336,12 @@ class Xsd11Element(XsdElement):
                     if value is not None:
                         attrib[k] = value
 
-                elem = etree_element(elem.tag, attrib=attrib)
+                elem = Element(elem.tag, attrib=attrib)
             else:
-                elem = etree_element(elem.tag)
+                elem = Element(elem.tag)
 
         if inherited:
-            dummy = etree_element('_dummy_element', attrib=inherited)
+            dummy = Element('_dummy_element', attrib=inherited)
             dummy.attrib.update(elem.attrib)
 
             for alt in self.alternatives:
