@@ -12,18 +12,20 @@ This module contains classes for XML Schema model groups.
 """
 import warnings
 from collections.abc import MutableMapping
+from copy import copy as _copy
 from typing import TYPE_CHECKING, overload, Any, Iterable, Iterator, List, \
     MutableSequence, Optional, Tuple, Union
+from xml.etree import ElementTree
 
 from .. import limits
 from ..exceptions import XMLSchemaValueError
 from ..names import XSD_GROUP, XSD_SEQUENCE, XSD_ALL, XSD_CHOICE, XSD_ELEMENT, \
     XSD_ANY, XSI_TYPE, XSD_ANY_TYPE, XSD_ANNOTATION
-from ..etree import etree_element, ElementData
 from ..aliases import ElementType, NamespacesType, SchemaType, IterDecodeType, \
     IterEncodeType, ModelParticleType, SchemaElementType, ComponentClassType
 from ..translation import gettext as _
 from ..helpers import get_qname, local_name, raw_xml_encode
+from ..converters import ElementData
 
 from .exceptions import XMLSchemaModelError, XMLSchemaModelDepthError, \
     XMLSchemaValidationError, XMLSchemaChildrenValidationError, \
@@ -37,7 +39,7 @@ from .models import ModelVisitor, distinguishable_paths
 if TYPE_CHECKING:
     from .complex_types import XsdComplexType
 
-ANY_ELEMENT = etree_element(
+ANY_ELEMENT = ElementTree.Element(
     XSD_ANY,
     attrib={
         'namespace': '##any',
@@ -344,26 +346,23 @@ class XsdGroup(XsdComponent, MutableSequence[ModelParticleType],
         group, children = self, iter(self)
 
         while True:
-            try:
-                child = next(children)
-            except StopIteration:
+            for child in children:
+                if child is item:
+                    _subgroups = [x[0] for x in subgroups]
+                    _subgroups.append(group)
+                    return _subgroups
+                elif isinstance(child, XsdGroup):
+                    if len(subgroups) > limits.MAX_MODEL_DEPTH:
+                        raise XMLSchemaModelDepthError(self)
+                    subgroups.append((group, children))
+                    group, children = child, iter(child)
+                    break
+            else:
                 try:
                     group, children = subgroups.pop()
                 except IndexError:
                     msg = _('{!r} is not a particle of the model group')
                     raise XMLSchemaModelError(self, msg.format(item)) from None
-                else:
-                    continue
-
-            if child is item:
-                _subgroups = [x[0] for x in subgroups]
-                _subgroups.append(group)
-                return _subgroups
-            elif isinstance(child, XsdGroup):
-                if len(subgroups) > limits.MAX_MODEL_DEPTH:
-                    raise XMLSchemaModelDepthError(self)
-                subgroups.append((group, children))
-                group, children = child, iter(child)
 
     def overall_min_occurs(self, item: ModelParticleType) -> int:
         """Returns the overall min occurs of a particle in the model."""
@@ -668,11 +667,7 @@ class XsdGroup(XsdComponent, MutableSequence[ModelParticleType],
         # Same model: declarations must simply preserve order
         other_iterator = iter(other.iter_model())
         for item in self.iter_model():
-            while True:
-                try:
-                    other_item = next(other_iterator)
-                except StopIteration:
-                    return False
+            for other_item in other_iterator:
                 if other_item is item or item.is_restriction(other_item, check_occurs):
                     break
                 elif other.model == 'choice':
@@ -686,6 +681,8 @@ class XsdGroup(XsdComponent, MutableSequence[ModelParticleType],
                         break
                 elif not other_item.is_emptiable():
                     return False
+            else:
+                return False
 
         if other.model != 'choice':
             for other_item in other_iterator:
@@ -1271,7 +1268,7 @@ class Xsd11Group(XsdGroup):
     Class for XSD 1.1 *model group* definitions.
 
     .. The XSD 1.1 model groups differ from XSD 1.0 groups for the 'all' model,
-    .. that can contains also other groups.
+       that can contains also other groups.
     ..  <all
           id = ID
           maxOccurs = (0 | 1) : 1
@@ -1406,7 +1403,7 @@ class Xsd11Group(XsdGroup):
                         w2.extended = True
                         break
                 else:
-                    wildcards.append(w1.copy())
+                    wildcards.append(_copy(w1))
 
         base_items.extend(w for w in wildcards if hasattr(w, 'extended'))
 
